'use strict';

var React = require('react');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var React__default = /*#__PURE__*/_interopDefaultLegacy(React);

const Context = React.createContext({
    focusInput: () => null,
    handleValuesChange: () => null,
    selectInputContent: () => null,
    setInputValue: () => null,
    validate: () => true,
});
function Provider({ children, autoFocus, defaultValue = '', onChange = () => null, onCompleted = () => null, type = 'number', }) {
    /**
     * generate a new array, map through it
     * and replace with the value when possible
     */
    const fillValues = React.useCallback((value) => new Array(children.length).fill('').map((_, index) => { var _a; return (_a = value[index]) !== null && _a !== void 0 ? _a : ''; }), [children.length]);
    const [values, setValues] = React.useState(fillValues(defaultValue));
    const refs = React.useMemo(() => new Array(children.length)
        .fill(null)
        .map(() => React.createRef()), [children.length]);
    const focusInput = React.useCallback((index) => {
        var _a;
        const input = (_a = refs[index]) === null || _a === void 0 ? void 0 : _a.current;
        if (input) {
            requestAnimationFrame(() => {
                input.focus();
            });
        }
    }, [refs]);
    const handleValuesChange = React.useCallback((input) => {
        setValues(fillValues(input));
        const isCompleted = input.length === length;
        if (isCompleted) {
            onCompleted(input);
            // blurInput();
        }
        focusInput(input.length);
    }, [fillValues, focusInput, onCompleted]);
    const selectInputContent = React.useCallback((index) => {
        var _a;
        const input = (_a = refs[index]) === null || _a === void 0 ? void 0 : _a.current;
        if (input) {
            requestAnimationFrame(() => {
                input.select();
            });
        }
    }, [refs]);
    const setInputValue = React.useCallback((value, index) => {
        const nextValues = [...values];
        nextValues[index] = value;
        setValues(nextValues);
        const stringifiedValues = nextValues.join('');
        const isLast = index === children.length - 1;
        console.log(index, children.length, index === children.length - 1);
        if (isLast) {
            onCompleted(stringifiedValues);
            return;
        }
        onChange(stringifiedValues);
    }, [children.length, onChange, onCompleted, values]);
    const validate = React.useCallback((input) => {
        if (type === 'number') {
            return /^\d/.test(input);
        }
        if (type === 'alphanumeric') {
            return /^[a-zA-Z0-9]/.test(input);
        }
        return true;
    }, [type]);
    const memoizedValue = React.useMemo(() => ({
        focusInput,
        handleValuesChange,
        selectInputContent,
        setInputValue,
        validate,
    }), [
        focusInput,
        handleValuesChange,
        selectInputContent,
        setInputValue,
        validate,
    ]);
    /**
     * autoFocus
     */
    React.useEffect(() => {
        if (autoFocus) {
            focusInput(0);
        }
    }, [autoFocus, focusInput]);
    return (React__default["default"].createElement(Context.Provider, { value: memoizedValue },
        React__default["default"].createElement(React__default["default"].Fragment, null, JSON.stringify(values, null, 2)),
        React.Children.map(children, (child, index) => React.cloneElement(child, {
            index,
            isLast: index === children.length - 1,
            value: values[index],
            ref: refs[index],
        }))));
}
function useInputVerificationCode() {
    const context = React.useContext(Context);
    if (!context) {
        throw new Error('useInputVerificationCode can only be used inside of a Provider');
    }
    return context;
}

function styleInject(css, ref) {
  if ( ref === void 0 ) ref = {};
  var insertAt = ref.insertAt;

  if (!css || typeof document === 'undefined') { return; }

  var head = document.head || document.getElementsByTagName('head')[0];
  var style = document.createElement('style');
  style.type = 'text/css';

  if (insertAt === 'top') {
    if (head.firstChild) {
      head.insertBefore(style, head.firstChild);
    } else {
      head.appendChild(style);
    }
  } else {
    head.appendChild(style);
  }

  if (style.styleSheet) {
    style.styleSheet.cssText = css;
  } else {
    style.appendChild(document.createTextNode(css));
  }
}

var css_248z = ".ReactInputVerificationCode-container {\n  display: flex;\n  gap: 0.5rem;\n}\n\n.ReactInputVerificationCode-item {\n  width: 4rem;\n  height: 4rem;\n  padding: 0;\n  border-radius: 4px;\n  font-size: 1.2rem;\n  text-align: center;\n  border: 0;\n  color: #555;\n  box-shadow: 0 0 0 1px #ccc;\n  outline: 0;\n  transition: box-shadow 0.2s ease-out;\n}\n\n.ReactInputVerificationCode-item:focus {\n  box-shadow: 0 0 0 2px #333;\n}\n";
styleInject(css_248z);

const Input = React.forwardRef(({ index, isLast, value }, ref) => {
    const { focusInput, handleValuesChange, selectInputContent, setInputValue, validate, } = useInputVerificationCode();
    // passed by the context
    if (handleValuesChange === undefined ||
        index === undefined ||
        isLast === undefined ||
        value === undefined) {
        return null;
    }
    const blurInput = () => {
        const input = ref.current;
        if (input) {
            input.blur();
        }
    };
    const onInputFocus = () => {
        selectInputContent(index);
    };
    const onInputKeyDown = (event) => {
        const eventKey = event.key;
        console.log({ event, eventKey, index });
        /**
         * handle "delete and go back" events
         */
        if (eventKey === 'Backspace' || eventKey === 'Delete') {
            setInputValue('', index);
            focusInput(index - 1);
            return;
        }
        /**
         * if the eventKey is not valid, don't go any further
         * and select the content of the input for a better UX
         */
        if (!validate(eventKey)) {
            selectInputContent(index);
            return;
        }
        setInputValue(eventKey, index);
        /**
         * if the input is the last of the list
         * blur it, otherwise focus the next one
         */
        if (isLast) {
            blurInput();
            return;
        }
        focusInput(index + 1);
    };
    const onInputChange = (event) => {
        const eventValue = event.target.value;
        console.log('onInputChange', { event, eventValue });
        /**
         * handle OTP and pasted codes
         */
        if (eventValue.length > 1) {
            handleValuesChange(eventValue);
        }
    };
    return (React__default["default"].createElement("input", { autoComplete: 'one-time-code', className: 'ReactInputVerificationCode-item', onChange: onInputChange, onFocus: onInputFocus, onKeyDown: onInputKeyDown, ref: ref, value: value }));
});

const ReactInputVerificationCode = (props) => (React__default["default"].createElement("div", { className: 'ReactInputVerificationCode-container' },
    React__default["default"].createElement(Provider, Object.assign({}, props))));
ReactInputVerificationCode.Input = Input;

module.exports = ReactInputVerificationCode;
